"""This is a sample metrics class used for testing & example sake."""
import sys
from typing import Optional
from mongo_tooling_metrics import get_hook, register_hook
from mongo_tooling_metrics.base_models import SubMetrics, TopLevelMetrics
from mongo_tooling_metrics.lib.hooks import ExitHook
from mongo_tooling_metrics.lib.sub_metrics import GitInfo, HostInfo


class BasicMetrics(TopLevelMetrics):
    """Collect basic metrics on the host machine."""
    host_info: HostInfo
    git_info: GitInfo
    exit_code: Optional[int]

    @classmethod
    def generate_metrics(cls) -> TopLevelMetrics:
        exit_hook = get_hook(ExitHook)
        return cls(
            host_info=HostInfo.generate_metrics(),
            git_info=GitInfo.generate_metrics('.'),
            exit_code=None if exit_hook.is_malformed() else exit_hook.exit_code,
        )

    @staticmethod
    def should_collect_metrics() -> bool:
        """Collect basic metrics at all times."""
        return True

    @staticmethod
    def initialize_hooks() -> None:
        """Initialize the exit hook bc the exit code is needed for these metrics."""
        sys.exit = register_hook(ExitHook(original_fn=sys.exit))

    def is_malformed(self) -> bool:
        """Return 'False' if the 'exit_code' is 'None' -- indicating the hook is malformed."""
        return self.exit_code == None



from __future__ import annotations
import threading
from typing import Dict, Type
from mongo_tooling_metrics.base_models import BaseHook

from mongo_tooling_metrics.errors import InvalidHookAccess


class _HookRegistry:

    registry: Dict = {}
    lock: threading.Lock = threading.Lock()

    def _safe_set_hook(self, hook_instance: BaseHook) -> bool:
        """Add the hook to the registry if it does not exist return the singleton hook instance."""
        with self.lock:
            self.registry.setdefault(hook_instance.__class__, hook_instance)
            return self.registry[hook_instance.__class__]

    def _safe_get_hook(self, hook_class: Type[BaseHook]) -> BaseHook:
        """Get the hook if it exists -- else fail."""
        try:
            with self.lock:
                return self.registry[hook_class]
        except KeyError as exc:
            raise InvalidHookAccess(
                f"Hook was accessed but never initialized: {hook_class}") from exc
